import re

class WindowManager(object):

    def __init__ (self):
        pass

    def find_window(self, class_name, window_name = None):
        pass

    def find_window_regex(self, regex):
        pass

    def set_foreground(self):
        """put the window in the foreground"""
        pass
